/* This is file 120x80x2.C, a program to reformat a text file into 2 text    */
/* columns of 120 rows by 80 character columns (max.) each.                  */

/* Coded in Microsoft C 5.1 for PC DOS by Daniel Ross,                       */
/*   starting July 5, 1985.                                                  */
/* Revised 9/11/88   13:20                                                   */

/* Copyright 1985 by Data Technology Corp., Santa Clara, CA 95051.  This     */
/* program may be redistributed so long as no charge is made for the program */
/* itself; nominal copy service charge, media charge, postage charge, etc.   */
/* acceptable.  Do not alter, delete, obscure, or fail to copy this          */
/* copyright and licensing notice.                                           */

/* Parameters expected in the DOS command line:                              */
/* 1.   Name of the ruler file.                                              */
/* 2.   Name of the input text file.                                         */
/* 3.   Name of the output text file.                                        */
/* 4.   An optional parameter consisting of the letter R, meaning print the  */
/*      ruler file in addition to the text file.                             */

/* The single record in the ruler file, and each record in the input text    */
/* file, must be no longer than 80 text characters (after Tab expansion)     */
/* followed by Carriage Return and Line Feed.                                */

/* The ruler file must contain exactly 1 record which contains no Tab        */
/* characters.  The record in the ruler file is used by this program to      */
/* determine the Tab stop columns in the input text file.  Each text         */
/* character may be either the letter T (upper case only), meaning that this */
/* column is a Tab stop column, or any other printing character (including   */
/* Space), which this program will not interpret.                            */

/* Limitations on program generality:                                        */
/* 1.   The ^Z which terminates the input file, must appear immediately      */
/*      after the CRLF which terminates a record, not in the middle of a     */
/*      record.                                                              */
/* 2.   There must not be any Form Feed characters in the input file.        */
/* 3.   The ruler file and the input text file must contain only ASCII       */
/*      characters; no characters with value greater than FF(hex).           */
/* 4.   The only ASCII control characters that this program interprets are   */
/*      those mentioned above.  All other ASCII control characters in the    */
/*      input text file are printed.  Note that some text editors may leave  */
/*      ASCII control characters in text files, to be interpreted in some    */
/*      manner peculiar to the text editor.                                  */

#include <STDLIB.H>
#include <STDIO.H>
#include <STRING.H>
#include <TIME.H>
#include <TYPES.H>
#include <STAT.H>

#define FALSE   0
#define TRUE    1
#define TorF    int

char                TextColSep [] = "  |  " ;

#define MaxInRec 80
#define TextCols 2
#define Term1 (MaxInRec * TextCols)
#define Term2 ((sizeof TextColSep) * (TextCols - 1))
#define MaxTextLines 120
#define FlagChar '\200'

char                * Cursor ;
struct stat         StatBuf ;
FILE                * RulerFile ;
FILE                * SourcFile ;
FILE                * DestFile ;
char                HeaderBuf           [120] = "File " ;
char                PageNrBuf           [10] ;
char                HeaderPageBuf [(sizeof HeaderBuf) + (sizeof PageNrBuf)] ;
time_t              Now ;
char                TimeString          [26] ;
char                ReorderedTimeString [26] ;
char                RecOfSpaces         [MaxInRec + 1] ;
char                RulerBuf            [MaxInRec + 4] ;
char                InBuf               [MaxInRec + 4] ;
char                TabExpBuf           [MaxInRec + 4] ;
char                OutBuf              [MaxTextLines] [Term1 + Term2 + 4] ;
char                Rulers              [Term1 + Term2 + 2] ;
char                * RulerCursor ;
char                * InCursor ;
char                * TabExpCursor ;
char                InChar ;
int                 TextLine ;
int                 InRecNr    = 0 ;
int                 OutRecNr   = 0 ;
int                 OutPageNr  = 0 ;
TorF                PrintRuler = FALSE ;
TorF                EOFWasSeen = FALSE ;
TorF                OKtoTab ;

/* Output is in ReorderedTimeString []                                       */
void   FormatDateTime (time_t * DateTime)
  {
    strcpy  (TimeString,   ctime (DateTime)) ;
    strncpy (ReorderedTimeString,   & TimeString [11],   9) ;
    strncpy ( & ReorderedTimeString [9],   TimeString,   10) ;
    ReorderedTimeString [19]   =   ',' ;
    strncpy ( & ReorderedTimeString [20],   & TimeString [19],   5) ;
    ReorderedTimeString [25]   =   0 ;
  }

void   PadSpaces (void)
  {
    for (Cursor = TabExpBuf ;   * Cursor != '\n' ;   ++ Cursor)   ;
    while (Cursor != & TabExpBuf [MaxInRec])   * (Cursor ++ ) = ' ' ;
    * Cursor = 0 ;
  }

void   ApendSep (void)
  {
    for (TextLine = 0 ;   TextLine < MaxTextLines ;   ++ TextLine)
      strcat (OutBuf [TextLine], TextColSep) ;
  }

void   ApendCRLF (void)
  {
    for (TextLine = 0 ;   TextLine < MaxTextLines ;   ++ TextLine)
      strcat (OutBuf [TextLine], "\n") ;
  }

void   Write1Rec (char * Buf)
  {
    ++ OutRecNr ;
    if (fputs (Buf, DestFile)   ==   EOF)
      {
        if (feof (DestFile)   ||   ferror (DestFile))
          {
            fprintf (stderr, "\nErr writing record %d:\n", OutRecNr) ;
            fprintf (stderr, Buf) ;
            exit (1) ;
          } ;
      } ;
  }

void   IfExpBufFull (void)
  {
    if (TabExpCursor == & TabExpBuf [MaxInRec])
      {
        fprintf
          (
            stderr ,
            "\nInput record %d is too long after Tab expansion:\n" ,
            InRecNr
          ) ;
        fprintf (stderr, InBuf) ;
        exit (1) ;
      } ;
  }

void   Fill1TextCol (void)
  {
    if (EOFWasSeen == FALSE)
      for (TextLine = 0 ;   TextLine < MaxTextLines ;   ++ TextLine)
        {
          if ( ! fgets (InBuf, MaxInRec + 3, SourcFile))
            {
              EOFWasSeen = TRUE ;
              break ;
            } ;
          ++ InRecNr ;
          if (InBuf [MaxInRec + 3] != FlagChar)
            {
              fprintf (stderr, "\nInput record %d too long:\n", InRecNr) ;
              fprintf (stderr, InBuf) ;
              exit (1) ;
            } ;
          /* Expand tabs                                                     */
          InCursor     = InBuf ;
          TabExpCursor = TabExpBuf ;
          RulerCursor  = RulerBuf ;
          OKtoTab      = TRUE ;
          while ((InChar = * (InCursor ++ ))   !=   '\n')
            if (InChar == '\t')
              {
                do
                  {
                    IfExpBufFull () ;
                    if (( ! OKtoTab)  ||  ( ! (* RulerCursor)))
                      {
                        fprintf
                          (
                            stderr ,
            "\nInput record %d has more Tabs than ruler has Tab stop cols:\n" ,
                            InRecNr
                          ) ;
                        fprintf (stderr, InBuf) ;
                        exit (1) ;
                      } ;
                    * (TabExpCursor ++ ) = ' ' ;
                  }
                  while ( * ( ++ RulerCursor)   !=   'T') ;
              }
            else
              {
                IfExpBufFull () ;
                * (TabExpCursor ++ ) = InChar ;
                if ( ! ( * (RulerCursor ++ )))   OKtoTab = FALSE ;
              } ;
          /* Pad with spaces                                                 */
          while (TabExpCursor   !=   & TabExpBuf [MaxInRec])
            * (TabExpCursor ++ ) = ' ' ;
          * TabExpCursor = 0 ;
          strcat (OutBuf [TextLine], TabExpBuf) ;
        } ;
    for (   ;   TextLine < MaxTextLines ;   ++ TextLine)
      strcat (OutBuf [TextLine], RecOfSpaces) ;
  }

void   main (int argc, char * argv [], char * envp [])
  {
    /* Initialize RecOfSpaces []                                             */
    for (Cursor = RecOfSpaces ;   Cursor != & RecOfSpaces [MaxInRec] ;   )
      * (Cursor ++ ) = ' ' ;
    * Cursor = 0 ;
    /* Verify proper DOS parameters                                          */
    if ((argc < 4) || (argc > 5))
      {
        fprintf (stderr, "\nRequired 3 <= number of DOS parameters <= 4\n") ;
        exit (1) ;
      } ;
    if (argc == 5)
      {
        if ((strcmp (argv [4], "R"))   &&   (strcmp (argv [4], "r")))
          {
            fprintf (stderr, "\nOptional last DOS param can only be R\n") ;
            exit (1) ;
          } ;
        PrintRuler = TRUE ;
      } ;
    /* Initialize HeaderBuf []                                               */
    strcat (HeaderBuf, argv [2]) ;
    if (stat (argv [2], & StatBuf))
      {
        fprintf (stderr, "Cannot find input text file %s", argv [2]) ;
        exit (1) ;
      } ;
    if ( ! (StatBuf.st_mode & S_IFREG))
      {
        fprintf (stderr, "%s is not a text file", argv [2]) ;
        exit (1) ;
      } ;
    strcat (HeaderBuf, ", last revised at ") ;
    FormatDateTime ( & StatBuf.st_mtime) ;
    strcat (HeaderBuf, ReorderedTimeString) ;
    strcat (HeaderBuf, ", listed at ") ;
    time ( & Now) ;
    FormatDateTime ( & Now) ;
    strcat (HeaderBuf, ReorderedTimeString) ;
    strcat (HeaderBuf, ";   Page ") ;
    /* Initial processing of the ruler record                                */
    if ( ! (RulerFile = fopen (argv [1], "rt")))
      {
        fprintf (stderr, "\nNo file or err opening %s\n", argv [1]) ;
        exit (1) ;
      } ;
    RulerBuf [MaxInRec + 3] = FlagChar ;
    if ( ! fgets (RulerBuf, MaxInRec + 3, RulerFile))
      {
        fprintf (stderr, "\nRuler file is empty\n") ;
        exit (1) ;
      } ;
    if (RulerBuf [MaxInRec + 3] != FlagChar)
      {
        fprintf (stderr, "\nRuler record too long:\n") ;
        fprintf (stderr, RulerBuf) ;
        exit (1) ;
      } ;
    for (Cursor = RulerBuf ;   * Cursor != '\n' ;   ++ Cursor)
      if (( * Cursor < ' ')   ||   ( * Cursor > '~'))
        {
          fprintf (stderr, "\nRuler record has nonprinting char.:\n") ;
          fprintf (stderr, RulerBuf) ;
          exit (1) ;
        } ;
    if (fgets (InBuf, MaxInRec + 3, RulerFile))
      {
        fprintf (stderr, "\nRuler file must contain exactly 1 record.") ;
        fprintf (stderr, "\n2nd record in ruler file is:\n") ;
        fprintf (stderr, InBuf) ;
        exit (1) ;
      } ;
    if (fclose (RulerFile))
      {
        fprintf (stderr, "\nErr closing file %s\n", argv [1]) ;
        exit (1) ;
      } ;
    /* Create a double concatenation of the ruler record                     */
    strcpy (TabExpBuf, RulerBuf) ;
    PadSpaces () ;
    strcpy (Rulers, TabExpBuf) ;    strcat (Rulers, TextColSep) ;
    strcat (Rulers, TabExpBuf) ;    strcat (Rulers, "\n") ;
    /* Open the input and output files                                       */
    if ( ! (SourcFile = fopen (argv [2], "rt")))
      {
        fprintf (stderr, "\nNo file or err opening %s\n", argv [2]) ;
        exit (1) ;
      } ;
    if ( ! (DestFile = fopen (argv [3], "wt")))
      {
        fprintf (stderr, "\nErr opening file %s\n", argv [3]) ;
        exit (1) ;
      } ;
    /* Main loop of processing text records                                  */
    InBuf [MaxInRec + 3] = FlagChar ;
    for (;;)
      {
        /* Write the header record                                           */
        strcpy (HeaderPageBuf, HeaderBuf) ;
        ++ OutPageNr ;
        sprintf (PageNrBuf, "%d\n", OutPageNr) ;
        strcat (HeaderPageBuf, PageNrBuf) ;
        Write1Rec (HeaderPageBuf) ;
        if (PrintRuler)
          {
            Write1Rec (Rulers) ;
          }
        else
          Write1Rec ("\n") ;
        Write1Rec ("\n") ;
        Write1Rec ("\n") ;
        for (TextLine = 0 ;   TextLine < MaxTextLines ;   ++ TextLine)
          OutBuf [TextLine] [0] = 0 ;
        do
          {
            Fill1TextCol () ;
            if (EOFWasSeen)   break ;
            ApendSep () ;
            Fill1TextCol () ;
          }
          while (FALSE) ;
        ApendCRLF () ;
        for (TextLine = 0 ;   TextLine < MaxTextLines ;   ++ TextLine)
          Write1Rec (OutBuf [TextLine]) ;
        if (PrintRuler)
          {
            Write1Rec ("\n") ;
            Write1Rec (Rulers) ;
          } ;
        /* The DOS PRINT command appends Form Feed to the file               */
        if (EOFWasSeen)   break ;
        if (fputc ('\f', DestFile)   ==   EOF)
          {
            if (ferror (DestFile))
              {
                fprintf
                  (
                    stderr ,
                    "\nErr writing Form Feed after output record %d\n" ,
                    OutRecNr
                  ) ;
                exit (1) ;
              } ;
          } ;
      } ;
    if (fclose (DestFile))
      fprintf (stderr, "\nErr closing file %s\n", argv [3]) ;
    exit (0) ;
  }

/* End of source file                                                        */
