/* Program to convert FLYSPECK to array initialization data in the Microsoft */
/* C language                                                                */

/* Coded in Microsoft C 5.1 by Daniel Ross, starting 11/2/87                 */
/* Revised 9/11/88    10:00                                                  */

/* The input data to this program, FLYSPECK, was created by Daniel Ross      */
/* starting 6/28/82, and entered into the public domain by the author at     */
/* that time.                                                                */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define ColsPerChar 7
#define RowsPerChar (9+3)               /* 9 rows above base line + 3 rows   */
                                        /* of descenders                     */

FILE   * InputFile, * OutputFile ;
char   InputFileName  []   =   "FLYSPECK.TXT" ;
char   OutputFileName []   =   "FLYSPECK.C" ;

char   Buf [ColsPerChar + 4] ;
char   InputChar ;

int    CharIndex, Row, Col, Acc ;

void   FormatErr (void)
  {
    printf
      ("\nFormat err, char %02x(hex), row %d up from bottom descender" ,
        CharIndex, Row) ;
    printf ("\nActual line read:  %s", Buf) ;
    exit (1) ;
  }

void   main (void)
  {
    if ( ! (InputFile = fopen (InputFileName, "rt")))
      {
        printf ("\nUnable to open input file %s", InputFileName) ;
        exit (1) ;
      } ;
    if ( ! (OutputFile = fopen (OutputFileName, "wt")))
      {
        printf ("\nUnable to open output file %s", OutputFileName) ;
        exit (1) ;
      } ;
    for (CharIndex = 0 ;   CharIndex != 128 ;   ++ CharIndex)
      {
        if ( ! fgets (Buf, ColsPerChar + 4, InputFile))
          {
            printf
              ("\nRead err or early end, at separator before char %02x(hex)",
                CharIndex) ;
            exit (1) ;
          } ;
        if (strcmp (Buf, "---------\n"))
          {
            printf
            ("\nExpecting separator before char %02x(hex), actually read:  %s",
                CharIndex, Buf) ;
            exit (1) ;
          } ;
        if ( ! fgets (Buf, ColsPerChar + 4, InputFile))
          {
            printf
              ("\nRead err or early end, at blank line before char %02x(hex)",
                CharIndex) ;
            exit (1) ;
          } ;
        fprintf (OutputFile, "        {") ;
        for (Row = RowsPerChar - 1 ;   Row >= 0 ;   -- Row)
          {
            if ( ! fgets (Buf, ColsPerChar + 4, InputFile))
              {
                printf
   ("\nRead err or early end, char %02x(hex), row %d up from bottom descender",
                    CharIndex, Row) ;
                exit (1) ;
              } ;
            if
              (
                (Buf [0]   !=   '"')
                  ||
                (Buf [ColsPerChar + 1]   !=   '"')
                  ||
                (Buf [ColsPerChar + 2]   !=   '\n')
              )
              FormatErr () ;
            Acc = 0 ;
            for (Col = 1 ;   Col <= ColsPerChar ;   ++ Col)
              {
                InputChar   =   Buf [Col] ;
                if (InputChar   ==   '*')     Acc   |=   1 << (8 - Col) ;
                else
                  {
                    if (InputChar   !=   ' ')     FormatErr () ;
                  } ;
              } ;
            fprintf (OutputFile, "0x%02x", Acc) ;
            if (Row)   fprintf (OutputFile, ",") ;
            else
              {
                fprintf (OutputFile, "}") ;
                if (CharIndex != 127)   fprintf (OutputFile, ",") ;
                fprintf (OutputFile, "\n") ;
              } ;
          } ;
        if ( ! fgets (Buf, ColsPerChar + 4, InputFile))
          {
            printf
              ("\nRead err or early end, at blank line after char %02x(hex)",
                CharIndex) ;
            exit (1) ;
          } ;
      } ;
    if (fclose (OutputFile))
      {
        printf ("\nErr closing output file %s", OutputFileName) ;
        exit (1) ;
      } ;
    if (fclose (InputFile))
      {
        printf ("\nErr closing input file %s", InputFileName) ;
        exit (1) ;
      } ;
    exit (0) ;
  }

/* End of file                                                               */
