/* Program to format font FLYSPECK to H-P LaserJet2 soft font (portrait)    */

/* Coded in Microsoft C 5.1 by Daniel Ross, starting 3/11/88                */
/* Revised 10/10/88    16:00                                                */

/* Although not explicitly stated in any H-P manual, this code is written    */
/* with the assumption that the MSB of a word is sent before the LSB.        */

#include <STDLIB.H>
#include <STDIO.H>
#include <STRING.H>
/* More files are included below                                             */

#define FLYSPECKColsPerChar         7
#define FLYSPECKRowsAboveBaseLine   9
#define FLYSPECKDescenderRows       3
#define FLYSPECKRowsPerChar (FLYSPECKRowsAboveBaseLine + FLYSPECKDescenderRows)
#define ColsSeparatingChars         2

int   FLYSPECK [128] [FLYSPECKRowsPerChar]   =
  {
#include "FLYSPECK.C"
  } ;

#define FALSE               0
#define TRUE                1
#define TorF                int

#define Escape              0x1B

int     CurrentCharIndex ;
int     CurrentCharPattern [FLYSPECKRowsPerChar] ;
int     BlankRowsOnTop ;
int     BlankRowsOnBottom ;
int     BlankColsOnLeft ;
int     BlankColsOnRight ;
int     PatternWidth_Dots ;
int     PatternWidth_Bytes ;
int     PatternHeight ;
int     TotalPatternBytes ;
FILE    * FileHandle ;
char    FileName []   =   "FLYSPECK.SFP" ;

void   CopyToCurrentCharPattern (void)
  {
    static int   RowIndex ;
    for (RowIndex = 0 ;   RowIndex < FLYSPECKRowsPerChar ;   ++ RowIndex)
      CurrentCharPattern [RowIndex]   =
        FLYSPECK [CurrentCharIndex] [RowIndex];
  }

void   CalcBlankRowsOnTop (void)
  {
    for
      (
        BlankRowsOnTop = 0
          ;
        BlankRowsOnTop < FLYSPECKRowsPerChar
          ;
        ++ BlankRowsOnTop
      )
      {
        if (CurrentCharPattern [BlankRowsOnTop])   return ;
      } ;
  }

void   CalcBlankRowsOnBottom (void)
  {
    for
      (
        BlankRowsOnBottom = 0
          ;
        BlankRowsOnBottom < FLYSPECKRowsPerChar
          ;
        ++ BlankRowsOnBottom
      )
      {
        if (CurrentCharPattern [(FLYSPECKRowsPerChar - 1) - BlankRowsOnBottom])
          return ;
      } ;
  }

void   CalcBlankColsOnLeft (void)
  {
    static int   Mask, RowIndex ;
    for
      (
        BlankColsOnLeft = 0
          ;
        BlankColsOnLeft < FLYSPECKColsPerChar
          ;
        ++ BlankColsOnLeft
      )
      {
        Mask   =   0x0080 >> BlankColsOnLeft ;
        for (RowIndex = 0 ;   RowIndex < FLYSPECKRowsPerChar ;   ++ RowIndex)
          {
            if (CurrentCharPattern [RowIndex] & Mask)   return ;
          } ;
      } ;
  }

void   CalcBlankColsOnRight (void)
  {
    static int   Mask, RowIndex ;
    for
      (
        BlankColsOnRight = 0
          ;
        BlankColsOnRight < FLYSPECKColsPerChar
          ;
        ++ BlankColsOnRight
      )
      {
        Mask   =   0x0080 >> ((FLYSPECKColsPerChar - 1) - BlankColsOnRight) ;
        for (RowIndex = 0 ;   RowIndex < FLYSPECKRowsPerChar ;   ++ RowIndex)
          {
            if (CurrentCharPattern [RowIndex] & Mask)   return ;
          } ;
      } ;
  }

void   JustifyTheCurrentCharPattern (void)
  {
    static int   RowIndex ;
    if (BlankRowsOnTop)
      {
        for
          (
            RowIndex = 0
              ;
            RowIndex < (FLYSPECKRowsPerChar - BlankRowsOnTop)
              ;
            ++ RowIndex
          )
          CurrentCharPattern [RowIndex]   =
            CurrentCharPattern [RowIndex + BlankRowsOnTop] ;
      } ;
    if (BlankColsOnLeft)
      {
        for
          (
            RowIndex = 0
              ;
            RowIndex < FLYSPECKRowsPerChar
              ;
            ++ RowIndex
          )
          CurrentCharPattern [RowIndex]   <<=   BlankColsOnLeft ;
      } ;
  }

void   CalcPatternSizeToSend (void)
  {
    PatternWidth_Dots      =
      FLYSPECKColsPerChar  - BlankColsOnLeft - BlankColsOnRight ;
    PatternWidth_Bytes     =   1 ;
    PatternHeight          =
      FLYSPECKRowsPerChar  - BlankRowsOnTop  - BlankRowsOnBottom ;
    TotalPatternBytes      =   PatternWidth_Bytes * PatternHeight ;
  }

void   Print1Byte (int TheByte)
  {
    fprintf (FileHandle, "%c", TheByte & 0xFF) ;
  }

void   Print1Word (int TheWord)
  {
    Print1Byte (TheWord >> 8) ;
    Print1Byte (TheWord) ;
  }

void   PrintTheFontDescriptor (void)
  {
    Print1Word (64) ;                   /* Font descriptor size              */
    Print1Word (0) ;                    /* Reserved byte & font type         */
    Print1Word (0) ;                    /* Reserved word                     */
    Print1Word (FLYSPECKRowsAboveBaseLine) ;
    Print1Word (FLYSPECKColsPerChar) ;
    Print1Word (FLYSPECKRowsPerChar) ;
    Print1Word (0) ;                    /* Orientation & spacing             */
    Print1Word (21) ;                   /* Symbol set                        */
    Print1Word ((FLYSPECKColsPerChar + ColsSeparatingChars) * 4) ;  /* Pitch */
    Print1Word ((300*10) / (192/4)) ;   /* Height -> 128 lines/page          */
    Print1Word (6*4) ;                  /* x height                          */
    Print1Word (0) ;                    /* Width type & style                */
    Print1Word (0) ;                    /* Stroke weight & typeface          */
    Print1Word (2) ;                    /* Reserved byte & serif style       */
    Print1Word (0) ;                    /* Reserved word                     */
    Print1Word (((-4)<<8) + 1) ;        /* Underline distance & height       */
    Print1Word ((300*10) / (192/4)) ;   /* Text height                       */
    Print1Word ((FLYSPECKColsPerChar + ColsSeparatingChars) * 4); /* T width */
    Print1Word (0) ;                    /* Reserved word                     */
    Print1Word (0) ;                    /* Reserved word                     */
    Print1Word (0) ;                    /* Pitch & height extended           */
    Print1Word (0) ;                    /* Reserved word                     */
    Print1Word (0) ;                    /* Reserved word                     */
    Print1Word (0) ;                    /* Reserved word                     */
    fprintf (FileHandle, "%16s", "Flyspeck        ") ;
  }

void   main (void)
  {
    static int      RowIndex ;
    if ( ! (FileHandle = fopen (FileName, "wb")))
      {
        printf ("\nUnable to open font file %s", FileName) ;
        exit (1) ;
      } ;
    /* Print the command that precedes the font descriptor                   */
    fprintf (FileHandle, "%c)s64W", Escape) ;
    PrintTheFontDescriptor () ;
    for
      (
        CurrentCharIndex =  0
          ;
        CurrentCharIndex != 128
          ;
        ++ CurrentCharIndex
      )
      {
        if (CurrentCharIndex == ' ')   continue ;       /* No space char.    */
        /* Print the command that selects the character                      */
        fprintf (FileHandle, "%c*c%dE", Escape, CurrentCharIndex) ;
        CopyToCurrentCharPattern        () ;
        CalcBlankRowsOnTop              () ;
        CalcBlankRowsOnBottom           () ;
        CalcBlankColsOnLeft             () ;
        CalcBlankColsOnRight            () ;
        JustifyTheCurrentCharPattern    () ;
        CalcPatternSizeToSend           () ;
        /* Print the command that precedes the character descriptor          */
        fprintf
          (
            FileHandle
              ,
            "%c(s%dW"
              ,
            Escape
              ,
            16 + TotalPatternBytes
          ) ;
        /* Print the character descriptor                                    */
        Print1Word ((4<<8) + 0) ;       /* Format & continuation             */
        Print1Word ((14<<8) + 1) ;      /* Descriptor size & class           */
        Print1Word (0) ;                /* Orientation & reserved byte       */
        Print1Word (BlankColsOnLeft) ;
        Print1Word (FLYSPECKRowsAboveBaseLine - BlankRowsOnTop) ;
        Print1Word (PatternWidth_Dots) ;
        Print1Word (PatternHeight) ;
        Print1Word ((FLYSPECKColsPerChar + ColsSeparatingChars) * 4) ; /* dX */
        for (RowIndex = 0 ;   RowIndex != PatternHeight ;   ++ RowIndex)
          Print1Byte (CurrentCharPattern [RowIndex]) ;
      } ;
    if (fclose (FileHandle))
      {
        printf ("\nError closing font file %s", FileName) ;
        exit (1) ;
      } ;
    exit (0) ;
  }

/* End of file                                                              */
