/* Program to format font MYOPIA for downloading into a H-P LaserJet2       */
/* printer                                                                  */

/* Coded in Microsoft C 5.1 by Daniel Ross, starting 3/11/88                */
/* Revised 10/10/88    16:00                                                */

/* Although not explicitly stated in any H-P manual, this code is written    */
/* with the assumption that the MSB of a word is sent before the LSB.        */

#include <STDLIB.H>
#include <STDIO.H>
#include <STRING.H>
#include <MATH.H>
/* More files are included below                                             */

#define MYOPIAColsPerChar         9
#define MYOPIARowsAboveBaseLine   13
#define MYOPIADescenderRows       4
#define MYOPIARowsPerChar (MYOPIARowsAboveBaseLine + MYOPIADescenderRows)
#define ColsSeparatingChars       2

int   MYOPIA [128] [MYOPIARowsPerChar]   =
  {
#include "MYOPIA.C"
  } ;

#define FALSE               0
#define TRUE                1
#define TorF                int

#define Escape              0x1B

#define FontIDNr            14

int     CurrentCharIndex ;
int     CurrentCharPattern [MYOPIARowsPerChar] ;
int     BlankRowsOnTop ;
int     BlankRowsOnBottom ;
int     BlankColsOnLeft ;
int     BlankColsOnRight ;
int     PatternWidth_Dots ;
int     PatternWidth_Bytes ;
int     PatternHeight ;
int     TotalPatternBytes ;
FILE    * FileHandle ;
char    FileName []   =   "MYOPIA.LOD" ;

void   CopyToCurrentCharPattern (void)
  {
    static int   RowIndex ;
    for (RowIndex = 0 ;   RowIndex < MYOPIARowsPerChar ;   ++ RowIndex)
      CurrentCharPattern [RowIndex]   =   MYOPIA [CurrentCharIndex] [RowIndex];
  }

void   CalcBlankRowsOnTop (void)
  {
    for
      (
        BlankRowsOnTop = 0
          ;
        BlankRowsOnTop < MYOPIARowsPerChar
          ;
        ++ BlankRowsOnTop
      )
      {
        if (CurrentCharPattern [BlankRowsOnTop])   return ;
      } ;
  }

void   CalcBlankRowsOnBottom (void)
  {
    for
      (
        BlankRowsOnBottom = 0
          ;
        BlankRowsOnBottom < MYOPIARowsPerChar
          ;
        ++ BlankRowsOnBottom
      )
      {
        if (CurrentCharPattern [(MYOPIARowsPerChar - 1) - BlankRowsOnBottom])
          return ;
      } ;
  }

void   CalcBlankColsOnLeft (void)
  {
    static int   Mask, RowIndex ;
    for
      (
        BlankColsOnLeft = 0
          ;
        BlankColsOnLeft < MYOPIAColsPerChar
          ;
        ++ BlankColsOnLeft
      )
      {
        Mask   =   0x8000 >> BlankColsOnLeft ;
        for (RowIndex = 0 ;   RowIndex < MYOPIARowsPerChar ;   ++ RowIndex)
          {
            if (CurrentCharPattern [RowIndex] & Mask)   return ;
          } ;
      } ;
  }

void   CalcBlankColsOnRight (void)
  {
    static int   Mask, RowIndex ;
    for
      (
        BlankColsOnRight = 0
          ;
        BlankColsOnRight < MYOPIAColsPerChar
          ;
        ++ BlankColsOnRight
      )
      {
        Mask   =   0x8000 >> ((MYOPIAColsPerChar - 1) - BlankColsOnRight) ;
        for (RowIndex = 0 ;   RowIndex < MYOPIARowsPerChar ;   ++ RowIndex)
          {
            if (CurrentCharPattern [RowIndex] & Mask)   return ;
          } ;
      } ;
  }

void   JustifyTheCurrentCharPattern (void)
  {
    static int   RowIndex ;
    if (BlankRowsOnTop)
      {
        for
          (
            RowIndex = 0
              ;
            RowIndex < (MYOPIARowsPerChar - BlankRowsOnTop)
              ;
            ++ RowIndex
          )
          CurrentCharPattern [RowIndex]   =
            CurrentCharPattern [RowIndex + BlankRowsOnTop] ;
      } ;
    if (BlankColsOnLeft)
      {
        for
          (
            RowIndex = 0
              ;
            RowIndex < MYOPIARowsPerChar
              ;
            ++ RowIndex
          )
          CurrentCharPattern [RowIndex]   <<=   BlankColsOnLeft ;
      } ;
  }

void   CalcPatternSizeToSend (void)
  {
    PatternWidth_Dots    =
      MYOPIAColsPerChar  - BlankColsOnLeft - BlankColsOnRight ;
    PatternWidth_Bytes   =   (PatternWidth_Dots > 8) ? 2 : 1 ;
    PatternHeight        =
      MYOPIARowsPerChar  - BlankRowsOnTop  - BlankRowsOnBottom ;
    TotalPatternBytes    =   PatternWidth_Bytes * PatternHeight ;
  }

void   Print1Byte (int TheByte)
  {
    fprintf (FileHandle, "%c", TheByte & 0xFF) ;
  }

void   Print1Word (int TheWord)
  {
    Print1Byte (TheWord >> 8) ;
    Print1Byte (TheWord) ;
  }

void   PrintTheFontDescriptor (void)
  {
    Print1Word (64) ;                   /* Font descriptor size              */
    Print1Word (0) ;                    /* Reserved byte & font type         */
    Print1Word (0) ;                    /* Reserved word                     */
    Print1Word (MYOPIARowsAboveBaseLine) ;
    Print1Word (MYOPIAColsPerChar) ;
    Print1Word (MYOPIARowsPerChar) ;
    Print1Word (0) ;                    /* Orientation & spacing             */
    Print1Word (21) ;                   /* Symbol set                        */
    Print1Word ((MYOPIAColsPerChar + ColsSeparatingChars) * 4) ;  /* Pitch   */
    Print1Word ((300*10) / (128/4)) ;   /* Height -> 128 lines/page          */
    Print1Word (8*4) ;                  /* x height                          */
    Print1Word (0) ;                    /* Width type & style                */
    Print1Word (0) ;                    /* Stroke weight & typeface          */
    Print1Word (2) ;                    /* Reserved byte & serif style       */
    Print1Word (0) ;                    /* Reserved word                     */
    Print1Word (((-6)<<8) + 1) ;        /* Underline distance & height       */
    Print1Word ((300*10) / (128/4)) ;   /* Text height                       */
    Print1Word ((MYOPIAColsPerChar + ColsSeparatingChars) * 4) ;  /* T width */
    Print1Word (0) ;                    /* Reserved word                     */
    Print1Word (0) ;                    /* Reserved word                     */
    Print1Word (0) ;                    /* Pitch & height extended           */
    Print1Word (0) ;                    /* Reserved word                     */
    Print1Word (0) ;                    /* Reserved word                     */
    Print1Word (0) ;                    /* Reserved word                     */
    fprintf (FileHandle, "%16s", "Myopia          ") ;
  }

void   main (void)
  {
    static int      RowIndex ;
    static long     PartialResult ;
    static double   VMI_InchesTimes48 ;
    if ( ! (FileHandle = fopen (FileName, "wb")))
      {
        printf ("\nUnable to open font file %s", FileName) ;
        exit (1) ;
      } ;
    /* Print the command to reset the printer                                */
    fprintf (FileHandle, "%cE", Escape) ;
    /* Print the command to select the font                                  */
    fprintf (FileHandle, "%c*c%dD", Escape, FontIDNr) ;
    /* Print the command that precedes the font descriptor                   */
    fprintf (FileHandle, "%c)s64W", Escape) ;
    PrintTheFontDescriptor () ;
    for
      (
        CurrentCharIndex =  0
          ;
        CurrentCharIndex != 128
          ;
        ++ CurrentCharIndex
      )
      {
        if (CurrentCharIndex == ' ')   continue ;       /* No space char.    */
        /* Print the command that selects the character                      */
        fprintf (FileHandle, "%c*c%dE", Escape, CurrentCharIndex) ;
        CopyToCurrentCharPattern        () ;
        CalcBlankRowsOnTop              () ;
        CalcBlankRowsOnBottom           () ;
        CalcBlankColsOnLeft             () ;
        CalcBlankColsOnRight            () ;
        JustifyTheCurrentCharPattern    () ;
        CalcPatternSizeToSend           () ;
        /* Print the command that precedes the character descriptor          */
        fprintf
          (
            FileHandle
              ,
            "%c(s%dW"
              ,
            Escape
              ,
            16 + TotalPatternBytes
          ) ;
        /* Print the character descriptor                                    */
        Print1Word ((4<<8) + 0) ;       /* Format & continuation             */
        Print1Word ((14<<8) + 1) ;      /* Descriptor size & class           */
        Print1Word (0) ;                /* Orientation & reserved byte       */
        Print1Word (BlankColsOnLeft) ;
        Print1Word (MYOPIARowsAboveBaseLine - BlankRowsOnTop) ;
        Print1Word (PatternWidth_Dots) ;
        Print1Word (PatternHeight) ;
        Print1Word ((MYOPIAColsPerChar + ColsSeparatingChars) * 4) ;  /* dX  */
        if (PatternWidth_Bytes == 1)
          {
            for (RowIndex = 0 ;   RowIndex != PatternHeight ;   ++ RowIndex)
              Print1Byte (CurrentCharPattern [RowIndex] >> 8) ;
          }
        else
          {
            for (RowIndex = 0 ;   RowIndex != PatternHeight ;   ++ RowIndex)
              Print1Word (CurrentCharPattern [RowIndex]) ;
          } ;
      } ;
    /* Print the command that makes the font permanent                       */
    fprintf (FileHandle, "%c*c5F", Escape) ;
    /* Print the command that makes the font to be the primary selected font */
    fprintf (FileHandle, "%c(%dX", Escape, FontIDNr) ;
    /* Print the command to set vertical motion index                        */
    PartialResult   =   ((long)10 * (long)48 * (long)10000) / (long)128 ;
    VMI_InchesTimes48   =   ((double) PartialResult) / 10000.0 ;
    fprintf (FileHandle, "%c&l%.4lfC", Escape, VMI_InchesTimes48) ;
    if (fclose (FileHandle))
      {
        printf ("\nError closing font file %s", FileName) ;
        exit (1) ;
      } ;
    exit (0) ;
  }

/* End of file                                                              */
